
#include <windows.h>  
#include "SetMapMd.h" 


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName  = "MyApp";
LPCTSTR lpszTitle    = "My Application"; 

BOOL RegisterWin95( CONST WNDCLASS* lpwc );

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static int nSave;

   switch( uMsg )
   {
      case WM_CREATE  : 
              {
                 HDC hDC = GetDC( hWnd );

                 // Initialize the device context.
                 //...............................
                 SetMapMode( hDC, MM_ISOTROPIC );
                 SetWindowExtEx( hDC, 1, 1, NULL );
                 SetViewportExtEx( hDC, 2, 2, NULL );
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                        {
                           HDC   hDC;
                           SIZE  sizeWExt;
                           SIZE  sizeVExt;
                           POINT pt;
                           char  szMsg[40];

                           hDC = GetDC( hWnd );

                           // Check to see if the mapping mode has
                           // changed.. If it has, restore it.
                           //.................................
                           if ( GetMapMode( hDC ) != MM_ISOTROPIC )
                              RestoreDC( hDC, nSave );

                           // Get the mapping mode ratio.
                           //............................
                           GetWindowExtEx( hDC, &sizeWExt );
                           GetViewportExtEx( hDC, &sizeVExt );

                           wsprintf( szMsg, "MM_ISOTROPIC - %d%%", 
                                     MulDiv( sizeVExt.cx, 100, sizeWExt.cx ));

                           // Determine where to place the text 
                           // for the MM_TEXT line.
                           //..................................   
                           pt.x = 110;
                           DPtoLP( hDC, &pt, 1 );
                            
                           // Draw the labels for the lines.
                           //...............................
                           TextOut( hDC, pt.x, 37, "MM_TEXT - 100%", 14 );
                           TextOut( hDC, 105, 17, szMsg, strlen( szMsg ) );
           
                           // Draw the MM_ISOTROPIC line.
                           //............................
                           MoveToEx( hDC, 0, 20, NULL );
                           LineTo( hDC, 100, 20 );

                           // Determine placement for the 
                           // MM_TEXT line.
                           //............................
                           pt.y = 40;
                           LPtoDP( hDC, &pt, 1 );

                           // Save the device context
                           // before changing it.
                           //........................                            
                           nSave = SaveDC( hDC );

                           // Draw the MM_TEXT line.
                           //.......................
                           SetMapMode( hDC, MM_TEXT );
                           MoveToEx( hDC, 0, pt.y, NULL );
                           LineTo( hDC, 100, pt.y );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
